<?php
namespace Setka\Workflow\Admin\MetaBoxes\WorkflowID;

use Korobochkin\WPKit\Options\OptionInterface;
use Korobochkin\WPKit\PostMeta\PostMetaInterface;
use Setka\Workflow\Plugin;
use Setka\Workflow\Prototypes\MetaBoxes\AbstractMetaBox;
use Setka\Workflow\Prototypes\MetaBoxes\TwigMetaBoxView;
use Setka\Workflow\Services\Account\Account;
use Setka\Workflow\Services\PostUtilities;
use Setka\Workflow\Services\WorkflowTicketUtilities;
use Setka\WorkflowSDK\Actions\Tickets\PublishTicketAction;
use Setka\WorkflowSDK\Actions\Tickets\UnpublishTicketAction;
use Setka\WorkflowSDK\Actions\Tickets\UpdateTicketAction;
use Setka\WorkflowSDK\API;

class WorkflowIDMetaBox extends AbstractMetaBox
{
    /**
     * @var Account
     */
    protected $account;

    /**
     * @var OptionInterface
     */
    protected $workflowPostTypesOption;

    /**
     * @var string Base Url to Setka API.
     */
    protected $baseUrl;

    /**
     * @var API
     */
    protected $api;

    /**
     * @var PostMetaInterface
     */
    protected $workflowTicketIdPostMeta;

    /**
     * @var OptionInterface
     */
    protected $publishAutomaticallyOption;

    /**
     * WorkflowIDMetaBox constructor.
     *
     * @param $workflowPostTypesOption OptionInterface
     * @param $baseUrl string Base url to Setka API.
     * @param $api API Workflow API.
     */
    public function __construct(
        OptionInterface $workflowPostTypesOption,
        $baseUrl,
        API $api
    ) {
        $this
            ->setWorkflowPostTypesOption($workflowPostTypesOption)
            ->setBaseUrl($baseUrl)
            ->setApi($api)

            ->setId(Plugin::_NAME_ . '_workflow_id')
            ->setTitle(_x('Setka Workflow', 'MetaBox title.', Plugin::NAME))
            ->setScreen($this->getWorkflowPostTypesOption()->get())
            ->setContext('side');

        $view = new TwigMetaBoxView();
        $view->setTemplate('admin/meta-boxes/workflow-id/workflow-id.html.twig');
        $this->setView($view);
    }

    /**
     * @inheritdoc
     */
    public function lateConstruct()
    {
        $this->setFormEntity(new WorkflowId());

        if($this->getRequest()->query->has('post')) {
            $postId = (int) $this->getRequest()->query->get('post');
        } elseif ($this->getRequest()->request->has('post_ID')) {
            $postId = (int) $this->getRequest()->request->get('post_ID');
        }

        if(isset($postId)) {
            $ticketId = (int) $this->getWorkflowTicketIdPostMeta()->setPostId($postId)->get();

            if ($ticketId > 0) {
                $ticketUtils = new WorkflowTicketUtilities($this->getBaseUrl(), $this->getAccount());

                $ticketUtils
                    ->setTicketId($ticketId)
                    ->generateUrl();

                $this->getFormEntity()->setUrl($ticketUtils->getUrlGenerated());
            }

            unset($postId, $ticketId);
        }

        $this->setForm(
            $this->getFormFactory()->createNamed(
                Plugin::_NAME_,
                WorkflowIdType::class,
                $this->getFormEntity()
            )
        );

        $this->handleRequest();

        $attributes = array(
            'metaBox' => $this,
            'form' => $this->getForm()->createView(),
            'translations' => array(
                'urlCaption' => __('Enter Setka Workflow Ticket Url', Plugin::NAME),
            ),
        );

        $this->getView()->setContext($attributes);

        return $this;
    }

    public function handleRequest()
    {
        // Handle form submission only if user allowed to edit current post.
        if (!current_user_can('edit_post', $this->getWorkflowTicketIdPostMeta()->getPostId())) {
            return $this;
        }

        $form = $this->getForm()->handleRequest($this->getRequest());

        if ($form->isSubmitted()) {
            if ($this->getWorkflowTicketIdPostMeta()->getPostId()) {
                if ($this->getFormEntity()->getUrl()) {
                    try {
                        $ticketUtils = new WorkflowTicketUtilities($this->getBaseUrl(), $this->getAccount());
                        $ticketId    = $ticketUtils
                            ->setUrl($this->getFormEntity()->getUrl())
                            ->parseUrl()
                            ->getTicketId();
                        $this->getWorkflowTicketIdPostMeta()->updateValue($ticketId);

                        // Update Workflow Ticket after WP set up the post status (published or draft).
                        add_action('post_updated', array($this, 'updateWorkflowTicket'), 10, 3);
                    } catch (\Exception $exception) {
                        // Do nothing.
                    }
                } else {
                    $this->getWorkflowTicketIdPostMeta()->delete();
                }
            }
        }
    }

    /**
     * Update ticket state.
     *
     * @param $action string 'publish' or 'unpublish' the ticket.
     *
     * @return $this For chain calls.
     */
    protected function updateTicketInfo($action = 'unpublish')
    {
        $updateTicketAction = new UpdateTicketAction($this->getApi());
        $actionDetails      = array(
            'space' => $this->getAccount()->getSpaceShortName(),
            'id' => $this->getWorkflowTicketIdPostMeta()->get(),
            'options' => array(
                'json' => array(
                    'title' => $this->getRequest()->request->get('post_title'),
                    'edit_post_url' => PostUtilities::getEditUrl($this->getWorkflowTicketIdPostMeta()->getPostId()),
                    'view_post_url' => PostUtilities::getViewUrl($this->getWorkflowTicketIdPostMeta()->getPostId()),
                ),
            ),
        );
        $actionDetails      = $updateTicketAction->configureDetails($actionDetails);
        $updateTicketAction->setDetails($actionDetails);
        unset($actionDetails);

        try {
            $updateTicketAction->request();
        } catch (\Exception $exception) {
            // Do nothing.
        }

        if(true === $this->getPublishAutomaticallyOption()->get()) {
            if($action == 'publish') {
                $publishUnPublishTicketAction = new PublishTicketAction($this->getApi());
            } else {
                $publishUnPublishTicketAction = new UnpublishTicketAction($this->getApi());
            }
            $actionDetails = $publishUnPublishTicketAction->configureDetails(
                array(
                    'space' => $this->getAccount()->getSpaceShortName(),
                    'id' => $this->getWorkflowTicketIdPostMeta()->get(),
                    'options' => array('json' => array())
                )
            );
            $publishUnPublishTicketAction->setDetails($actionDetails);
            unset($actionDetails);

            try {
                $publishUnPublishTicketAction->request();
            } catch (\Exception $exception) {
                // Do nothing.
            }
        }

        return $this;
    }

    /**
     * Update Workflow ticket after WP update the post.
     *
     * @param $id int Post Id.
     * @param $postAfter \WP_Post
     * @param $postBefore \WP_Post
     *
     * @return $this For chain calls.
     */
    public function updateWorkflowTicket($id, $postAfter, $postBefore)
    {
        if ($this->getWorkflowTicketIdPostMeta()->getPostId() !== $postAfter->ID) {
            return $this;
        }

        if ($postAfter->post_status == 'publish') {
            $this->updateTicketInfo('publish');
        } else {
            $this->updateTicketInfo('unpublish');
        }

        return $this;
    }

    /**
     * @return Account
     */
    public function getAccount()
    {
        return $this->account;
    }

    /**
     * @param Account $account
     *
     * @return $this
     */
    public function setAccount(Account $account)
    {
        $this->account = $account;
        return $this;
    }

    /**
     * @return OptionInterface
     */
    public function getWorkflowPostTypesOption()
    {
        return $this->workflowPostTypesOption;
    }

    /**
     * @param OptionInterface $workflowPostTypesOption
     *
     * @return $this
     */
    public function setWorkflowPostTypesOption(OptionInterface $workflowPostTypesOption)
    {
        $this->workflowPostTypesOption = $workflowPostTypesOption;
        return $this;
    }

    /**
     * @return PostMetaInterface
     */
    public function getWorkflowTicketIdPostMeta()
    {
        return $this->workflowTicketIdPostMeta;
    }

    /**
     * @param PostMetaInterface $workflowTicketIdPostMeta
     *
     * @return $this
     */
    public function setWorkflowTicketIdPostMeta($workflowTicketIdPostMeta)
    {
        $this->workflowTicketIdPostMeta = $workflowTicketIdPostMeta;
        return $this;
    }

    /**
     * @return string
     */
    public function getBaseUrl()
    {
        return $this->baseUrl;
    }

    /**
     * @param string $baseUrl
     *
     * @return $this
     */
    public function setBaseUrl($baseUrl)
    {
        $this->baseUrl = $baseUrl;
        return $this;
    }

    /**
     * @return API
     */
    public function getApi()
    {
        return $this->api;
    }

    /**
     * @param API $api
     *
     * @return $this
     */
    public function setApi($api)
    {
        $this->api = $api;
        return $this;
    }

    /**
     * @return OptionInterface
     */
    public function getPublishAutomaticallyOption()
    {
        return $this->publishAutomaticallyOption;
    }

    /**
     * @param OptionInterface $publishAutomaticallyOption
     *
     * @return $this
     */
    public function setPublishAutomaticallyOption($publishAutomaticallyOption)
    {
        $this->publishAutomaticallyOption = $publishAutomaticallyOption;
        return $this;
    }
}
